% VFI_SL.m
% Uses Bi-section to find equilibrium R 
% Need as inputs candidate for R below and above -- can run with equal values 
% JG, UJJ, LS 2016

clear

RLH0 =  [ 0.037203   0.037203];     %  equal values set maxit =1; run just once

maxit = 1;

Rite= zeros(maxit+1,2);
tarBv=zeros(maxit,1);           % Target Indicator for Bi-section
Rite(1,:) = RLH0;               % iterated R vector

for jj=1:maxit

R = (Rite(jj,1)+Rite(jj,2))/2;

tau =  0.400;                 
delta=  0.025;              
lambda=  .05 ;                
bbeta=  0.99;               
mubar=  0.01;               
cp=  exp(mubar)/bbeta-1;    
xik = .29;                  

et4 = 0;
et1 = 0.6815 ;                

%Distribution
zl = -1;
zh = -zl;
et2 = -3/5*et4*(zh^2);
et3 = 3*(0.5 - et1*zh)/(zh^3);   
et0 = 0.5+et4*(zh^4)/20;

% Grid for leverage used for omega and omega_prime
% Pick a small grid range around the steady state
grismid=.00005;         
grisout= grismid*20;         
lomid = .40;    
upmid = .44;

omega = [  [lomid: grisout: lomid-grisout]  [lomid : grismid :upmid]   [upmid+grisout :grisout:.55] ]';

nw = max(size(omega));
omega_p = omega;       
index_opt_p = ones(size(omega_p));    

maxiterp = 1000;
dispiter = 1001;


q = ones(size(omega_p)) ; 
v = 1-delta*(1-tau) - (1-lambda)*q.*omega/exp(mubar);
v_p = v;    

zs = max(min(R - (cp + lambda/(1-tau))*omega_p/exp(mubar) + v_p/(1-tau),1),-1);  

phi = et1 + et2*zs + et3*zs.^2 + et4*zs.^3;
Phi = et0 + et1*zs + (et2*zs.^2)/2 + (et3*zs.^3)/3 + (et4*zs.^4)/4;
Ez_tail = ( et1*(zh^2)/2 + et2*(zh.^3)/3+ et3*(zh.^4)/4 + et4*(zh.^5)/5 ) -( et1*(zs.^2)/2 + et2*(zs.^3)/3+ et3*(zs.^4)/4 + et4*(zs.^5)/5 );
NodefR_p = Phi.*( R - (cp + lambda/(1-tau))*omega_p/exp(mubar) + v_p/(1-tau) ) + Ez_tail; 

err = 1; err1 = 1; err2 = 1;

up_v = 1;
up_q= .7;

Tv = v;

for iterp=1:maxiterp;

    for w = 1:nw
         [Tv(w), index_opt_p(w)] = max(q.*(omega_p-(1-lambda)*omega(w)/exp(mubar)) - delta*(1-tau) + (1-tau)*bbeta*NodefR_p);
    end
    err1 = max(abs(v - Tv));
    v = up_v*Tv+(1-up_v)*v;
    
    v_p=v;
    zs = max(min(R - (cp + lambda/(1-tau))*omega_p/exp(mubar) + v_p/(1-tau),1),-1);      % Cutoff

    phi = et1 + et2*zs + et3*zs.^2 + et4*zs.^3;
    Phi = et0 + et1*zs + (et2*zs.^2)/2 + (et3*zs.^3)/3 + (et4*zs.^4)/4;
    Ez_tail = ( et1*(zh^2)/2 + et2*(zh.^3)/3+ et3*(zh.^4)/4 + et4*(zh.^5)/5 ) ...
            -( et1*(zs.^2)/2 + et2*(zs.^3)/3+ et3*(zs.^4)/4 + et4*(zs.^5)/5 );
    NodefR_p = Phi.*( R - (cp + lambda/(1-tau))*omega_p/exp(mubar) + v_p/(1-tau) ) + Ez_tail; 
     
    qfut = q(index_opt_p);
    Tq = bbeta*(Phi*(cp+lambda)/exp(mubar)+(1-lambda)*qfut/exp(mubar) - ((1-tau)*Ez_tail - (1-Phi).*((1-tau)*R - xik + v_p))./omega_p);
    err2 = max(abs(q - Tq));   
    
    q = up_q*Tq+(1-up_q)*q;
    
    err1v(iterp)=err1;
    err2v(iterp)=err2;
    if max(err1,err2) < 0.000001
        break
    end
   
    % smooth out discreteness in price function
    smooth=1;
    if smooth==1
    smo1=50;
    smo2=100;
    wei=ones(size(q));
    wei(1:10)=1*ones(10,1);
    uopt=fitoptions('Method', 'LinearLeastSquares','Weights',wei);
    if iterp>smo1 && iterp <=smo2
        qint=fit(omega_p, q, 'poly4',uopt);
        q =qint(omega_p);
    elseif iterp >smo2
         qint=fit(omega_p, q, 'poly6',uopt);
           q =qint(omega_p);
    end
    end
end

recom=1;
if recom==1;
for w = 1:nw
        Tv(w) = max(q.*(omega_p-(1-lambda)*omega(w)/exp(mubar)) - delta*(1-tau) + (1-tau)*bbeta*NodefR_p);
        index_opt_p(w) = find(Tv(w) == q.*(omega_p-(1-lambda)*omega(w)/exp(mubar)) - delta*(1-tau) + (1-tau)*bbeta*NodefR_p);
end
end
omega_opt = omega_p(index_opt_p);


issi = find(   abs(omega_opt - omega_p) ==  min(abs(omega_opt - omega_p)));
iss=issi;
if max(size(issi))>1
    iss=min(issi);
end

Tfirm= v(iss)+omega(iss)*q(iss)*(1-lambda)/exp(mubar);
[Tfirm  1-(1-tau)*delta ( Tfirm -( 1-(1-tau)*delta))];

mktl_ss=omega_p(iss)*q(iss) 
def_ss= (1-Phi(iss))

R0=R+.0001;
R_ss = fsolve(@(x) ifR(x,q(iss),omega_p(iss),v_p(iss),tau,bbeta,cp,lambda,mubar,et1,et4),R0);      

tarB=R_ss-R;
tarBv(jj)=tarB
if tarB>=0
    Rite(jj+1,1) = R;
    Rite(jj+1,2) = Rite(jj,2);
elseif  tarB <0 
    Rite(jj+1,2) = R;
    Rite(jj+1,1) = Rite(jj,1);
end

if jj==maxit
    break
end
end

